import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsHomepagePrefix(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.homepageprefix'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'HOMEPAGE = "ftp://isn\'t/21stcentury/tech"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'HOMEPAGE = "abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'HOMEPAGE = "ssh://won\'t/get/u/anywhere"',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.homepageprefix'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bbappend':
                                     'HOMEPAGE = "http://fancy.org"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'HOMEPAGE = "https://fancy.org"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'HOMEPAGE = " https://evenmore.fancy.org"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass

# -- known order --
# 'do_fetch',
# 'do_unpack',
# 'do_patch',
# 'do_configure',
# 'do_compile',
# 'do_install',
# 'do_populate_sysroot',
# 'do_build',
# 'do_package'


class TestClassOelintTaskOrder(TestBaseClass):

    def __generate_sample_code(self, first, second):
        return '''
            {first}() {{
                :
            }}
            {second}() {{
                :
            }}
            '''.format(first=first, second=second)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_fetch'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('pair', [
        ('do_unpack', 'do_fetch'),
        ('do_patch', 'do_fetch'),
        ('do_configure', 'do_fetch'),
        ('do_compile', 'do_fetch'),
        ('do_install', 'do_fetch'),
        ('do_populate_sysroot', 'do_fetch'),
        ('do_build', 'do_fetch'),
        ('do_package', 'do_fetch'),
    ])
    def test_bad_fetch(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_unpack'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('pair', [
        ('do_patch', 'do_unpack'),
        ('do_configure', 'do_unpack'),
        ('do_compile', 'do_unpack'),
        ('do_install', 'do_unpack'),
        ('do_populate_sysroot', 'do_unpack'),
        ('do_build', 'do_unpack'),
        ('do_package', 'do_unpack'),
    ])
    def test_bad_unpack(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_patch'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('pair', [
        ('do_configure', 'do_patch'),
        ('do_compile', 'do_patch'),
        ('do_install', 'do_patch'),
        ('do_populate_sysroot', 'do_patch'),
        ('do_build', 'do_patch'),
        ('do_package', 'do_patch'),
    ])
    def test_bad_patch(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_configure'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('pair', [
        ('do_compile', 'do_configure'),
        ('do_install', 'do_configure'),
        ('do_populate_sysroot', 'do_configure'),
        ('do_build', 'do_configure'),
        ('do_package', 'do_configure'),
    ])
    def test_bad_configure(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_compile'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('pair', [
        ('do_install', 'do_compile'),
        ('do_populate_sysroot', 'do_compile'),
        ('do_build', 'do_compile'),
        ('do_install', 'do_compile'),
        ('do_package', 'do_compile'),
    ])
    def test_bad_compile(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_install'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('pair', [
        ('do_populate_sysroot', 'do_install'),
        ('do_build', 'do_install'),
        ('do_package', 'do_install'),
    ])
    def test_bad_install(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_populate_sysroot'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('pair', [
        ('do_build', 'do_populate_sysroot'),
        ('do_package', 'do_populate_sysroot'),
    ])
    def test_bad_populate_sysroot(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_build'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('pair', [
        ('do_package', 'do_build'),
    ])
    def test_bad_build(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

####

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_fetch'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('pair', [
        ('do_fetch', 'do_unpack'),
        ('do_fetch', 'do_patch'),
        ('do_fetch', 'do_configure'),
        ('do_fetch', 'do_compile'),
        ('do_fetch', 'do_install'),
        ('do_fetch', 'do_populate_sysroot'),
        ('do_fetch', 'do_build'),
        ('do_fetch', 'do_package'),
    ])
    def test_good_fetch(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_unpack'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('pair', [
        ('do_unpack', 'do_patch'),
        ('do_unpack', 'do_configure'),
        ('do_unpack', 'do_compile'),
        ('do_unpack', 'do_install'),
        ('do_unpack', 'do_populate_sysroot'),
        ('do_unpack', 'do_build'),
        ('do_unpack', 'do_package'),
    ])
    def test_good_unpack(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_patch'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('pair', [
        ('do_patch', 'do_configure'),
        ('do_patch', 'do_compile'),
        ('do_patch', 'do_install'),
        ('do_patch', 'do_populate_sysroot'),
        ('do_patch', 'do_build'),
        ('do_patch', 'do_package'),
    ])
    def test_good_patch(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_configure'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('pair', [
        ('do_configure', 'do_compile'),
        ('do_configure', 'do_install'),
        ('do_configure', 'do_populate_sysroot'),
        ('do_configure', 'do_build'),
        ('do_configure', 'do_package'),
    ])
    def test_good_configure(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_compile'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('pair', [
        ('do_compile', 'do_install'),
        ('do_compile', 'do_populate_sysroot'),
        ('do_compile', 'do_build'),
        ('do_compile', 'do_install'),
        ('do_compile', 'do_package'),
    ])
    def test_good_compile(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_install'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('pair', [
        ('do_install', 'do_populate_sysroot'),
        ('do_install', 'do_build'),
        ('do_install', 'do_package'),
    ])
    def test_good_install(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_populate_sysroot'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('pair', [
        ('do_populate_sysroot', 'do_build'),
        ('do_populate_sysroot', 'do_package'),
    ])
    def test_good_populate_sysroot(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_build'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('pair', [
        ('do_build', 'do_package'),
    ])
    def test_good_build(self, id_, occurrence, pair):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(pair[0], pair[1]),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_build'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_build() {
                                         :
                                     }
                                     do_package() {
                                         :
                                     }
                                     ''',
                                 },
                             ],
                             )
    def test_good_pattern(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.order.do_build'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.inc':
                                     '''
                                     do_populate_sysroot() {
                                         :
                                     }
                                     do_package() {
                                         :
                                     }
                                     ''',
                                     'oelint_adv_test.bb':
                                     '''
                                     require oelint_adv_test.inc
                                     do_build() {
                                         :
                                     }
                                     ''',
                                 },
                             ],
                             )
    def test_single_file_scope(self, id_, occurrence, input_):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintBBClassUnderscore(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.bbclass.underscores'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'classes/oelint-test.bbclass':
                                     'VAR = "1"',
                                     'conf/layer.conf':
                                     '',
                                     'oelint_test.bb':
                                     'inherit oelint-test',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.bbclass.underscores'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'classes/oelint_test.bbclass':
                                     'VAR = "1"',
                                     'conf/layer.conf':
                                     '',
                                     'oelinttest.bb':
                                     'inherit oelint_test',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



from typing import List, Tuple

from oelint_parser.cls_item import Variable, Inherit
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class VarSRCURIAppend(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.vars.srcuriappend',
                         severity='error',
                         message='<FOO>')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        inherits = stash.GetItemsFor(filename=_file, classifier=Inherit.CLASSIFIER)
        if not inherits:
            return res

        items: List[Variable] = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER,
                                                  attribute=Variable.ATTR_VAR, attributeValue='SRC_URI')
        for item in items:
            if item.VarOp.strip() in ['+=']:
                override_delimiter = item.OverrideDelimiter
                res += self.finding(item.Origin, item.InFileLine,
                                    'Use SRC_URI{od}append otherwise this will override weak defaults by inherit'.format(
                                        od=override_delimiter))
        return res



from typing import List, Tuple

from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule
from oelint_adv.state import State


class FileNotApplicableInlineSuppression(Rule):
    def __init__(self, state: State = None):
        super().__init__(id='oelint.file.inlinesuppress_na',
                         severity='info',
                         message='Inline suppression for \'{id}\' is not needed')
        self._state = state

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        # special handling from main
        # do not announce any finding
        return []



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsMultilineIdent(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.multilineident'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "\\
                                         a \\
                                     b \\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     D = "a \\
                                     e \\
                                         "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     D = "    a \\
                                          e \\
                                         "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "a \\
                                       b \\
                                          c \\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "\\
                                         b \\
                                          c \\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A:append = "a \\
                                              b \\
                                                 c \\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A += "a \\
                                        b \\
                                           c \\
                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.multilineident'])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     D = "a \\
                                      e \\
                                         "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     D = "    a \\
                                         e \\
                                         "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "a \\
                                       b \\
                                          c \\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A:append = "a \\
                                              b \\
                                                 c \\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A += "a \\
                                        b \\
                                           c \\
                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_fix(self, input_, id_):
        self.fix_and_check(self._create_args_fix(input_), id_)

    @pytest.mark.parametrize('id_', ['oelint.vars.multilineident'])
    @pytest.mark.parametrize('occurrence', [2])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "a \\
                                     b \\
                                     c \\
                                         "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     D = "a \\
                                     e \\
                                     f \\
                                         "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "\\
                                     a \\
                                     b \\
                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_bad_two(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.multilineident'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "\\
                                         a \\
                                         b \\
                                         e \\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "a \\
                                          b \\
                                          c \\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A:append = "a \\
                                                 b \\
                                                 c \\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A += "a \\
                                           b \\
                                           c \\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "\\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "\\
                                         some \\"quoted\\" value \\
                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import os

datapath = os.path.dirname(__file__)


def has_data_file(release: str) -> bool:
    return os.path.exists(os.path.join(datapath, f'{release}.json'))


def known_variable_mod(release: str) -> str:
    if has_data_file(release):
        return os.path.join(datapath, f'{release}.json')
    return ''



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsNotNeededSpace(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.notneededspace'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR = " a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR += " a"',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.notneededspace'])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR = " a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR += " a"',
                                 },
                             ],
                             )
    def test_fix(self, input_, id_):
        self.fix_and_check(self._create_args_fix(input_), id_)

    @pytest.mark.parametrize('id_', ['oelint.vars.notneededspace'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR = "a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR += "a"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.notneededspace'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR_append = " a"',
                                 },
                             ],
                             )
    def test_good_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)



from typing import List, Tuple

from oelint_parser.cls_item import Inherit
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class VarInheritLast(Rule):
    def __init__(self) -> None:
        self._needles = [
            'cross',
            'native',
            'nativesdk',
        ]
        super().__init__(id='oelint.var.inheritlast',
                         severity='warning',
                         message='{VAR} needs to be inherited last',
                         appendix=self._needles)

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items: List[Inherit] = stash.GetItemsFor(filename=_file, classifier=Inherit.CLASSIFIER)
        _findings = []
        for i in items:
            for class_ in i.get_items():
                _findings.append((class_, i))

        _findings_class_only = [x[0] for x in _findings]
        for needle in self._needles:
            if needle in _findings_class_only and _findings[-1][0] != needle:
                try:
                    index = next(i for i, v in enumerate(_findings) if v[0] == needle)  # pragma: no cover
                    res += self.finding(_findings[index][1].Origin, _findings[index][1].InFileLine,
                                        self.Msg.format(VAR=needle),
                                        appendix=needle)
                except StopIteration:  # pragma: no cover
                    continue  # pragma: no cover
        return res



import argparse
import json
import os
from argparse import ArgumentTypeError

import pytest  # noqa: I900

from .base import TestBaseClass


# flake8: noqa S101 - n.a. for test files
class TestClassCustomRules(TestBaseClass):

    CUSTOM_RULES_DIR = os.path.join(os.path.dirname(__file__), 'customrules')

    def test_custom_rules_loading(self, capsys):
        from oelint_adv.__main__ import print_rulefile

        _args = self._create_args(
            {}, extraopts=[f'--customrules={TestClassCustomRules.CUSTOM_RULES_DIR}', '--print-rulefile'])
        print_rulefile(_args)

        out = json.loads(capsys.readouterr().out)

        assert 'test.rule.magic' in out

    def test_custom_rules_till(self, capsys):
        from oelint_adv.__main__ import print_rulefile

        _args = self._create_args(
            {}, extraopts=[
                '--release=rocko',
                f'--customrules={TestClassCustomRules.CUSTOM_RULES_DIR}',
                '--print-rulefile']
        )
        print_rulefile(_args)

        out = json.loads(capsys.readouterr().out)

        assert 'test.rule.magic.validtill_sumo' in out

    def test_custom_rules_till_na(self, capsys):
        from oelint_adv.__main__ import print_rulefile

        _args = self._create_args(
            {}, extraopts=[
                '--release=kirkstone',
                f'--customrules={TestClassCustomRules.CUSTOM_RULES_DIR}',
                '--print-rulefile']
        )
        print_rulefile(_args)

        out = json.loads(capsys.readouterr().out)

        assert 'test.rule.magic.validtill_sumo' not in out

    def test_custom_rules_from(self, capsys):
        from oelint_adv.__main__ import print_rulefile

        _args = self._create_args(
            {}, extraopts=[
                '--release=sumo',
                f'--customrules={TestClassCustomRules.CUSTOM_RULES_DIR}',
                '--print-rulefile']
        )
        print_rulefile(_args)

        out = json.loads(capsys.readouterr().out)

        assert 'test.rule.magic.validfrom_kirkstone' not in out

    def test_custom_rules_from_na(self, capsys):
        from oelint_adv.__main__ import print_rulefile

        _args = self._create_args(
            {}, extraopts=[
                '--release=kirkstone',
                f'--customrules={TestClassCustomRules.CUSTOM_RULES_DIR}',
                '--print-rulefile']
        )
        print_rulefile(_args)

        out = json.loads(capsys.readouterr().out)

        assert 'test.rule.magic.validfrom_kirkstone' in out

    def test_custom_rules_range(self, capsys):
        from oelint_adv.__main__ import print_rulefile

        _args = self._create_args(
            {}, extraopts=[
                '--release=dunfell',
                f'--customrules={TestClassCustomRules.CUSTOM_RULES_DIR}',
                '--print-rulefile']
        )
        print_rulefile(_args)

        out = json.loads(capsys.readouterr().out)

        assert 'test.rule.magic.validtill_kirkstone_from_thud' in out

    def test_custom_rules_range_na_1(self, capsys):
        from oelint_adv.__main__ import print_rulefile

        _args = self._create_args(
            {}, extraopts=[
                '--release=mickledore',
                f'--customrules={TestClassCustomRules.CUSTOM_RULES_DIR}',
                '--print-rulefile']
        )
        print_rulefile(_args)

        out = json.loads(capsys.readouterr().out)

        assert 'test.rule.magic.validtill_kirkstone_from_thud' not in out

    def test_custom_rules_range_na_2(self, capsys):
        from oelint_adv.__main__ import print_rulefile

        _args = self._create_args(
            {}, extraopts=[
                '--release=sumo',
                f'--customrules={TestClassCustomRules.CUSTOM_RULES_DIR}',
                '--print-rulefile']
        )
        print_rulefile(_args)

        out = json.loads(capsys.readouterr().out)

        assert 'test.rule.magic.validtill_kirkstone_from_thud' not in out

    def test_custom_rules_appendix(self, capsys):
        from oelint_adv.__main__ import print_rulefile

        _args = self._create_args(
            {}, extraopts=[
                '--release=dunfell',
                f'--customrules={TestClassCustomRules.CUSTOM_RULES_DIR}',
                '--print-rulefile']
        )
        print_rulefile(_args)

        out = json.loads(capsys.readouterr().out)

        assert 'test.rule.magic.specappendix.A' in out
        assert 'test.rule.magic.specappendix.B' in out

    def test_custom_rules_appendix_kirkstone(self, capsys):
        from oelint_adv.__main__ import print_rulefile

        _args = self._create_args(
            {}, extraopts=[
                '--release=kirkstone',
                f'--customrules={TestClassCustomRules.CUSTOM_RULES_DIR}',
                '--print-rulefile']
        )
        print_rulefile(_args)

        out = json.loads(capsys.readouterr().out)

        assert 'test.rule.magic.specappendix.A' in out
        assert 'test.rule.magic.specappendix.B' in out
        assert 'test.rule.magic.specappendix.kirkstone' in out



from typing import List, Tuple

from oelint_parser.cls_item import Inherit
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class VarNativeFilename(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.var.nativefilename',
                         severity='warning',
                         message='native-recipe-files should include \'-native\' in file name')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items: List[Inherit] = [x for x in stash.GetItemsFor(
            filename=_file, classifier=Inherit.CLASSIFIER) if 'native' in x.get_items()]
        if any(items):
            if _file.find('-native') == -1:
                res += self.finding(_file, 0)
        return res



from typing import List, Tuple

from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class VarPkgSpecific(Rule):
    def __init__(self) -> None:
        self.needles = ['RDEPENDS', 'RRECOMMENDS', 'RSUGGESTS', 'RCONFLICTS', 'RPROVIDES', 'RREPLACES',
                        'FILES', 'pkg_preinst', 'pkg_postinst', 'pkg_prerm', 'pkg_postrm', 'ALLOW_EMPTY']
        super().__init__(id='oelint.vars.pkgspecific',
                         severity='error',
                         message='Variable {VAR} is package-specific and therefore it should be {VAR}{DEL}<known package name>',
                         appendix=self.needles)

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []

        if self.is_lone_append(stash, _file):
            return res

        _packages = stash.GetValidPackageNames(_file)
        items: List[Variable] = stash.GetItemsFor(filename=_file,
                                                  classifier=Variable.CLASSIFIER,
                                                  attribute=Variable.ATTR_VAR,
                                                  attributeValue=self.needles)
        if not items:
            return res
        delimiter = items[-1].OverrideDelimiter
        for i in items:
            _machine = []
            if i.GetMachineEntry():
                _machine = [i.GetMachineEntry(), stash.ExpandTerm(_file, i.GetMachineEntry())]
            if not _machine or not any(x in _packages for x in _machine):
                res += self.finding(i.Origin, i.InFileLine,
                                    override_msg=self.Msg.format(self.Msg, VAR=i.VarName, DEL=delimiter), appendix=i.VarName)
        return res



from typing import List, Tuple

from oelint_parser.cls_item import Comment, Item
from oelint_parser.cls_stash import Stash
from oelint_parser.rpl_regex import RegexRpl

from oelint_adv.cls_rule import Rule


class NoSpaceRuleCont(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.spaces.linecont',
                         severity='error',
                         message='No spaces after line continuation')

    def __getMatches(self, _file: str, stash: Stash) -> List[Item]:
        res = []
        items = stash.GetItemsFor(filename=_file)
        for i in items:
            if isinstance(i, Comment):
                continue
            if i.Raw:  # pragma: no cover
                if RegexRpl.search(r'\\\s+\n', i.Raw):
                    res.append(i)
        return res

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        for i in self.__getMatches(_file, stash):
            res += self.finding(i.Origin, i.InFileLine)
        return res

    def fix(self, _file: str, stash: Stash) -> List[str]:
        res = []
        for i in self.__getMatches(_file, stash):
            i.RealRaw = RegexRpl.sub(r'\\\s+\n', '\\\n', i.RealRaw)
            i.Raw = RegexRpl.sub(r'\\\s+\n', '\\\n', i.Raw)
            res.append(_file)
        return res



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsDependsOrdered(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.dependsordered'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "zzz \\
                                     xyz"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "foo"
                                     DEPENDS += "bar"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN} += "zzz \\
                                                 xyz"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN} += "foo"
                                     RDEPENDS:${PN} += "bar"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.dependsordered'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "bar"
                                     DEPENDS += "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "xyz \\
                                                 zzz"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN} += "bar"
                                     RDEPENDS:${PN} += "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN} += "xyz \\
                                                 zzz"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS:class-native += "\\
                                         rubygems-mini-portile2-native \\
                                         rubygems-racc-native \\
                                     "
                                     DEPENDS:class-target += "\\
                                         rubygems-mini-portile2 \\
                                     "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "a (>= 1.2.3) \\
                                                 b (>= 4.5.6)"
                                     ''',
                                 },
                                 {
                                     'recipes/oelint_adv_test.bb':
                                     '''
                                     DEPENDS:append = " z"
                                     inherit foo
                                     ''',
                                     'conf/layer.conf':
                                     ' ',
                                     'classes/foo.bbclass':
                                     'DEPENDS:append = " a"',
                                 },
                                 {
                                     'recipes/oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN}:append = " z"
                                     inherit foo
                                     ''',
                                     'conf/layer.conf':
                                     ' ',
                                     'classes/foo.bbclass':
                                     'RDEPENDS:${PN}:append = " a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "abc def ghi"
                                     DEPENDS:remove = "def"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN} += "abc def ghi"
                                     RDEPENDS:${PN}:remove = "def"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN} = "x y z"
                                     RDEPENDS:${PN}:append:magic-machine = " a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN} = "x y z"
                                     RDEPENDS:${PN}:prepend:magic-machine = "a "
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS_${PN} += "\\
                                        bash \\
                                        libFormatConversion.so()(64bit) \\
                                        libMvCameraControl.so()(64bit) \\
                                        libMVRender.so()(64bit) \\
                                        libstdc++ \\
                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsDuplicate(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.duplicate'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS = "foo"
                                     DEPENDS += "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN} = "foo"
                                     RDEPENDS:${PN}:prepend = " foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN}-dev = "foo"
                                     RDEPENDS:${PN}-dev:prepend = " foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN}:class-target = "foo"
                                     RDEPENDS:${PN}:append:class-target = "foo "
                                     ''',
                                 },
                                 {
                                     'oelint-adv-test_1.0.bb': '',
                                     'dynamic-layers/a/oelint-adv-test_1.0.bbappend':
                                     '''
                                     DEPENDS += "a"
                                     ''',
                                     'dynamic-layers/a/oelint-adv-test_%.bbappend':
                                     '''
                                     DEPENDS += "a"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.duplicate'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS = "foo"
                                     DEPENDS_append = " foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS = "foo"
                                     DEPENDS_prepend = " foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS_${PN} = "foo"
                                     RDEPENDS_${PN}_prepend = " foo"
                                     ''',
                                 },
                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.duplicate'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "foo"
                                     DEPENDS = "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "${@inline.block}"
                                     DEPENDS += "${@inline.block}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "a (>= 1.2.3)"
                                     DEPENDS += "b (>= 1.2.3)"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN}:class-target = "foo"
                                     RDEPENDS:${PN} = "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb': '',
                                     'dynamic-layers/a/oelint_adv_test.bbappend':
                                     '''
                                     DEPENDS += "a"
                                     ''',
                                     'dynamic-layers/b/oelint_adv_test.bbappend':
                                     '''
                                     DEPENDS += "a"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.duplicate'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "foo"
                                     DEPENDS_class-native += "foo"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "foo"
                                     DEPENDS_remove = "foo"
                                     ''',
                                 },
                             ],
                             )
    def test_good_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarSRCURIWildcard(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.var.srcuriwildcard'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "file://*"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "git://foo.org;name=${PV} \\
                                         file://somedir/*"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.srcuriwildcard'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "git://foo.org;name=${PV}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "${@["", "file://init.cfg"][(d.getVar(\'VIRTUAL-RUNTIME_init_manager\') == \'busybox\')]}"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsRenamed(TestBaseClass):

    def _generate_code(self, var):
        return {'oelint_test.bb': f'{var} = "1"'}

    @pytest.mark.parametrize('id_', ['oelint.vars.renamed'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', ['PNBLACKLIST',
                                     'CVE_CHECK_PN_WHITELIST',
                                     'CVE_CHECK_WHITELIST',
                                     'MULTI_PROVIDER_WHITELIST',
                                     'SDK_LOCAL_CONF_BLACKLIST',
                                     'SDK_LOCAL_CONF_WHITELIST',
                                     'SDK_INHERIT_BLACKLIST',
                                     'SSTATE_DUPWHITELIST',
                                     'SYSROOT_DIRS_BLACKLIST',
                                     'UNKNOWN_CONFIGURE_WHITELIST',
                                     'ICECC_USER_CLASS_BL',
                                     'ICECC_SYSTEM_CLASS_BL',
                                     'ICECC_USER_PACKAGE_WL',
                                     'ICECC_USER_PACKAGE_BL',
                                     'ICECC_SYSTEM_PACKAGE_BL',
                                     'INHERIT_BLACKLIST',
                                     'TUNEABI_WHITELIST',
                                     'LICENSE_FLAGS_WHITELIST',
                                     'TCLIBCAPPEND',
                                     'VOLATILE_LOG_DIR',
                                     'VOLATILE_TMP_DIR',
                                     'WHITELIST_GPL-3.0-only',
                                     'WHITELIST_GPL-3.0-or-later',
                                     'WHITELIST_LGPL-3.0-only',
                                     'WHITELIST_LGPL-3.0-or-later'])
    def test_bad(self, var, id_, occurrence):
        self.check_for_id(self._create_args(self._generate_code(var)), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.renamed'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', ['PNBLACKLIST',
                                     'CVE_CHECK_PN_WHITELIST',
                                     'CVE_CHECK_WHITELIST',
                                     'MULTI_PROVIDER_WHITELIST',
                                     'SDK_LOCAL_CONF_BLACKLIST',
                                     'SDK_LOCAL_CONF_WHITELIST',
                                     'SDK_INHERIT_BLACKLIST',
                                     'SSTATE_DUPWHITELIST',
                                     'SYSROOT_DIRS_BLACKLIST',
                                     'UNKNOWN_CONFIGURE_WHITELIST',
                                     'ICECC_USER_CLASS_BL',
                                     'ICECC_SYSTEM_CLASS_BL',
                                     'ICECC_USER_PACKAGE_WL',
                                     'ICECC_USER_PACKAGE_BL',
                                     'ICECC_SYSTEM_PACKAGE_BL',
                                     'INHERIT_BLACKLIST',
                                     'TUNEABI_WHITELIST',
                                     'LICENSE_FLAGS_WHITELIST',
                                     'TCLIBCAPPEND',
                                     'VOLATILE_LOG_DIR',
                                     'VOLATILE_TMP_DIR',
                                     'WHITELIST_GPL-3.0-only',
                                     'WHITELIST_GPL-3.0-or-later',
                                     'WHITELIST_LGPL-3.0-only',
                                     'WHITELIST_LGPL-3.0-or-later'])
    def test_bad_older_release(self, var, id_, occurrence):
        self.check_for_id(self._create_args(self._generate_code(var), ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.renamed'])
    @pytest.mark.parametrize('var', ['PNBLACKLIST',
                                     'CVE_CHECK_PN_WHITELIST',
                                     'CVE_CHECK_WHITELIST',
                                     'MULTI_PROVIDER_WHITELIST',
                                     'SDK_LOCAL_CONF_BLACKLIST',
                                     'SDK_LOCAL_CONF_WHITELIST',
                                     'SDK_INHERIT_BLACKLIST',
                                     'SSTATE_DUPWHITELIST',
                                     'SYSROOT_DIRS_BLACKLIST',
                                     'UNKNOWN_CONFIGURE_WHITELIST',
                                     'ICECC_USER_CLASS_BL',
                                     'ICECC_SYSTEM_CLASS_BL',
                                     'ICECC_USER_PACKAGE_WL',
                                     'ICECC_USER_PACKAGE_BL',
                                     'ICECC_SYSTEM_PACKAGE_BL',
                                     'LICENSE_FLAGS_WHITELIST',
                                     'WHITELIST_GPL-3.0-only',
                                     'WHITELIST_GPL-3.0-or-later',
                                     'WHITELIST_LGPL-3.0-only',
                                     'WHITELIST_LGPL-3.0-or-later'])
    def test_fix(self, var, id_):
        self.fix_and_check(self._create_args_fix(self._generate_code(var)), id_)

    @pytest.mark.parametrize('id_', ['oelint.vars.renamed'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', ['INHERIT_BLACKLIST',
                                     'TUNEABI_WHITELIST',
                                     'TCLIBCAPPEND',
                                     'VOLATILE_LOG_DIR',
                                     'VOLATILE_TMP_DIR'])
    def test_bad_unfixable(self, var, id_, occurrence):
        self.check_for_id(self._create_args_fix(self._generate_code(var)), id_, occurrence)



import os

import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsPathHardcode(TestBaseClass):

    def __generate_sample_code(self, var):
        return '''
            VAR = "{var}"
            '''.format(var=var)

    @pytest.mark.parametrize('id_', ['oelint.vars.pathhardcode'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('pair', [
        ('${systemd_user_unitdir}', '/usr/lib/systemd/user'),
        ('${systemd_system_unitdir}', '/lib/systemd/system'),
        ('${docdir}', '/usr/share/doc'),
        ('${infodir}', '/usr/share/info'),
        ('${mandir}', '/usr/share/man'),
        ('${libexecdir}', '/usr/libexec'),
        ('${systemd_unitdir}', '/lib/systemd'),
        ('${libdir}', '/usr/lib'),
        ('${bindir}', '/usr/bin'),
        ('${datadir}', '/usr/share'),
        ('${includedir}', '/usr/include'),
        ('${localstatedir}', '/var'),
        ('${nonarch_base_libdir}', '/lib'),
        ('${sbindir}', '/usr/sbin'),
        ('${servicedir}', '/srv'),
        ('${sharedstatedir}', '/com'),
        ('${sysconfdir}', '/etc'),
    ])
    def test_bad(self, id_, occurrence, pair):
        id_ += '.{var}'.format(var=pair[0].strip('${}'))  # noqa: P103 - false positive
        for variation in [pair[1],
                          pair[1] + '/',
                          os.path.join(pair[1], 'fooooo/ggsg'),
                          os.path.join(pair[1], '*'),
                          os.path.join('${D}', pair[1])]:
            input_ = {
                'oelint_adv_test.bb': self.__generate_sample_code(variation),
            }
            self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.pathhardcode'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('pair', [
        ('${systemd_user_unitdir}', '/usr/lib/systemd/user'),
        ('${systemd_system_unitdir}', '/lib/systemd/system'),
        ('${docdir}', '/usr/share/doc'),
        ('${infodir}', '/usr/share/info'),
        ('${mandir}', '/usr/share/man'),
        ('${libexecdir}', '/usr/libexec'),
        ('${systemd_unitdir}', '/lib/systemd'),
        ('${libdir}', '/usr/lib'),
        ('${bindir}', '/usr/bin'),
        ('${datadir}', '/usr/share'),
        ('${includedir}', '/usr/include'),
        ('${localstatedir}', '/var'),
        ('${nonarch_base_libdir}', '/lib'),
        ('${nonarch_libdir}', '/usr/lib'),
        ('${oldincludedir}', '/usr/include'),
        ('${sbindir}', '/usr/sbin'),
        ('${servicedir}', '/srv'),
        ('${sharedstatedir}', '/com'),
        ('${sysconfdir}', '/etc'),
    ])
    def test_good(self, id_, occurrence, pair):
        id_ += '.{var}'.format(var=pair[0].strip('${}'))  # noqa: P103 - false positive
        for variation in [pair[0],
                          pair[0] + '/',
                          os.path.join(pair[0], 'fooooo/ggsg'),
                          os.path.join(pair[0], '*'),
                          os.path.join('${D}', pair[0])]:
            input_ = {
                'oelint_adv_test.bb': self.__generate_sample_code(variation),
            }
            self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.pathhardcode'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES_${PN} += "/usr/lib/totally.valid.file"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES_${PN} += "/usr/bin/totally.valid.file"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_install_append() {
                                         #To remove the default files from /etc/somefolder
                                         rm -f ${D}${sysconfdir}/somefolder/*
                                         install -m 0644 ${S}/usr/lib/* ${D}${libdir}
                                         install -m 0644 ${WORKDIR}/usr/bin ${D}${bindir}
                                         echo "foo" | sed "s#/usr/bin/python#/usr/bin/env python#g" > ${D}${bindir}/foo
                                     }
                                     ''',
                                 },
                             ],
                             )
    def test_good_pattern(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)
